@@ -1,14 +1,28 @@
 #include "common.c"
 
-/* Linux uses ptrace() */
-#if !defined(HAVE_PTRACE) || !defined(HAVE_SYS_PTRACE_H) || !defined(HAVE_SYS_USER_H)
+/* Linux uses ptrace().  We require PTRACE_SETOPTIONS so the exec tracing logic
+ * is sane.  Otherwise we need a lot of arch-specific hacks to make it work.
+ * This should be fine for linux-2.6+ versions.
+ */
+#if !defined(HAVE_PTRACE) || !defined(HAVE_SYS_PTRACE_H) || \
+    !defined(HAVE_SYS_USER_H) || !defined(PTRACE_SETOPTIONS)
 # define SB_NO_TRACE_ARCH
+#elif defined(__alpha__)
+# include "alpha.c"
+#elif defined(__arm__)
+# include "arm.c"
 #elif defined(__bfin__)
 # include "bfin.c"
 #elif defined(__hppa__)
 # include "hppa.c"
 #elif defined(__i386__)
 # include "i386.c"
+#elif defined(__ia64__)
+# include "ia64.c"
+#elif defined(__powerpc__)
+# include "powerpc.c"
+#elif defined(__s390__)
+# include "s390.c"
 #elif defined(__sparc__)
 # include "sparc.c"
 #elif defined(__x86_64__)
